import arcpy
import pandas as pd
import os
import datetime


# Author J.Connolly USEPA

# Description: Generate list of species in union file if the output from the previous isn't complete.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.


in_location = 'path\CriticalHabitat\CH_SpGroup_Union_Final_20190812.gdb'
file_suffix_clean = '_Union_Final_20190812'
out_csv = r'path\CriticalHabitat\CH_Species_included_inUnion' + \
          file_suffix_clean + '.csv'

start_time = datetime.datetime.now()
print "Start Time: " + start_time.ctime()

out_df = pd.DataFrame(index=(list(range(0, 1000))))
arcpy.env.workspace = in_location
fc_list = arcpy.ListFeatureClasses()

for fc in fc_list:
    group = fc.split("_")[1]
    att_array = arcpy.da.TableToNumPyArray((in_location + os.sep + fc), ['ZoneSpecies'])
    att_df = pd.DataFrame(data=att_array)
    att_df['ZoneSpecies'] = att_df['ZoneSpecies'].apply(
        lambda x: x.replace('[', '').replace(']', '').replace('u', '').replace(' ', '').replace("'", ""))
    spl = att_df['ZoneSpecies'].str.split(',', expand=True)
    out_list = []
    for col in spl.columns.values.tolist():
        out_list.extend(spl[col].values.tolist())
    group_spe = list(set(out_list))

    count = len(group_spe)
    remaining = [None] * (1000 - count)  # all columns need to 1000 rows - makes additional rows with value none
    merge_list = group_spe + remaining
    series_sp = pd.Series(merge_list)
    out_df[group] = series_sp.values

out_df.to_csv(out_csv)
end = datetime.datetime.now()
print "End Time: " + end.ctime()
elapsed = end - start_time
print "Elapsed  Time: " + str(elapsed)
