import arcpy
import os
import datetime

# Author J.Connolly USEPA

# Description: Clip union fc to land

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

# ingdb from Step 2
inlocation = r'path\file_name.gdb'
# file to be use as clip
clip_fc = r'path\Boundaries.gdb\Regions_dissolve'

# outlocation and suffix to be added to fc filename
outlocation = 'out locations\R_Clipped_Union_[date].gdb'
Clipped_suffix = "_ClippedRegions_[date]"


# ###Functions


def create_gdb(out_folder, out_name, out_path):
    # Create a new GDB
    if not arcpy.Exists(out_path):
        arcpy.CreateFileGDB_management(out_folder, out_name, "CURRENT")
        print 'Created GDB {0}'.format(out_name)


def clip_feature(in_location, clip_fc_in, out_location, clipped_suffix):
    # runs clip on files in in location
    clip_time = datetime.datetime.now()
    arcpy.Delete_management("ClipFeatures")
    arcpy.MakeFeatureLayer_management(clip_fc_in, "ClipFeatures")
    xy_tolerance = ""
    # Set workspace location
    arcpy.env.workspace = in_location
    fclist = arcpy.ListFeatureClasses()
    # loop through all files and run intersect
    for fc in fclist:
        out_feature = out_location + os.sep + fc + clipped_suffix
        in_features = in_location + os.sep + fc
        try:
            if not arcpy.Exists(out_feature):
                print "\nAdding ZoneID for {0}".format(fc)
                arcpy.AddField_management(fc, "ZoneID", "DOUBLE")
                with arcpy.da.UpdateCursor(fc, ['OBJECTID', 'ZoneID']) as cursor:
                    for row in cursor:
                        row[1] = row[0]
                        cursor.updateRow(row)
                print "Clipping {0}".format(fc)
                arcpy.Delete_management("inFeatures")
                arcpy.MakeFeatureLayer_management(in_features, "inFeatures")
                arcpy.Clip_analysis("inFeatures", "ClipFeatures", out_feature, xy_tolerance)
                print 'Clipped CompFile {0} took {1}'.format(fc, datetime.datetime.now() - clip_time)
            else:
                print ('Already clipped {0}'.format(fc))
                arcpy.AddField_management(fc, "ZoneID", "DOUBLE")

        except Exception as error:
            print(error.args[0])
            arcpy.Delete_management(out_feature)


start_time = datetime.datetime.now()
print "Start Time: " + start_time.ctime()

if not arcpy.Exists(outlocation):
    path, gdb_file = os.path.split(outlocation)
    create_gdb(path, gdb_file, outlocation)
clip_feature(inlocation, clip_fc, outlocation, Clipped_suffix)

end = datetime.datetime.now()
print "End Time: " + end.ctime()
elapsed = end - start_time
print "Elapsed  Time: " + str(elapsed)
